<?php
/*
Plugin Name: LePress
Plugin URI: http://trac.htk.tlu.ee/lepress/
Version: 2.0
Author: <a href="mailto:raido357@gmail.com">Raido Kuli</a>
Description: A plugin for a <a href="http://trac.htk.tlu.ee/lepress/">Centre for the Educational Technology</a>.
*/

require_once('config.php');
require_once('class-lepress-request.php');
require_once('service.php');

class LePress {

    function __construct() {
    	//load translation file
		load_plugin_textdomain(lepress_textdomain, false, basename(lepress_abspath).'/languages');
		
		//Add action/filter hooks
		add_action('admin_menu', array(&$this, 'createOptionsMenus'));
		add_action('admin_init', array(&$this, 'registerDefaultSettings'));
		add_action('admin_notices', array(&$this, 'checkIsProfileFilled'));
		add_filter('pre_update_option_lepress-settings', array(&$this, 'filterOptionChange'), 10, 3);
		register_deactivation_hook( __FILE__, array(&$this, 'uninstall'));
		//This constant is globally readable
		global $wpdb;
		define('wpdb_prefix', $wpdb->prefix);
		//Init service
		$this->service = new LePressService();
		//Load widget
		require_once('widget.php');
    }
	
	function filterOptionChange($option, $oldvalue, $newvalue) {
		if(!isSet($option['user-role']['student']) && $this->isStudentFeatures()) {
			//This means user has been in role "student" and now disabling it
			global $LePressStudent;
			$LePressStudent->initUninstall(); //Unsubscribes from all the courses, rest of the data is kept intact;
		}
		
		if(!isSet($option['user-role']['teacher']) && $this->isTeacherFeatures()) {
			//This means user has been in role "teacher" and now disabling it
			global $LePressTeacher;
			$LePressTeacher->initUninstall(); //Remove all subscriptions, rest of the data is kept intact
		}
		if(isSet($option['lepress-simple-password'])) {
			if(empty($option['lepress-simple-password'])) {
			 	unset($option['lepress-simple-password']);
			} else {
				file_put_contents('/tmp/pede.log', $newvalue['lepress-simple-password']." - ".$option['lepress-simple-password']."\n");
				if($oldvalue['lepress-simple-password'] != $option['lepress-simple-password']) {
					$option['lepress-simple-password'] = md5($option['lepress-simple-password']);
				}
			}
		}
		return $option;
	}
	
    function registerDefaultSettings() {
    	register_setting('lepress-settings', 'lepress-settings');
    	add_settings_section('lepress-general', '', array(&$this, 'generalSectionHeader'), 'lepress');
    	add_settings_field('lepress-user-role', __("Choose your role", lepress_textdomain), array(&$this, 'user_role_input'), 'lepress', 'lepress-general');
    	if($this->isTeacherFeatures() || $this->isStudentFeatures()) {
    		add_settings_field('lepress-display-names', __("Display names as", lepress_textdomain), array(&$this, 'display_names_input'), 'lepress', 'lepress-general');
    	}
    }

	function getBlogOwnerUser() {
		$blog_owner_email = get_bloginfo('admin_email');
		$blog_owner = get_user_by('email', $blog_owner_email);
		return $blog_owner;
	}
	function checkIsProfileFilled() {
		$blog_owner = $this->getBlogOwnerUser();
		$current_user = wp_get_current_user();
		$access_granted = false;
		foreach(get_users() as $user) {
			if($user->ID == $current_user->ID) {
				$access_granted = true;
				break;
			}
		}
		if((empty($current_user->first_name) || empty($current_user->last_name)) && $access_granted) {
			$this->echoNoticeDiv(''.__('Please edit your profile and fill <b>Firstname</b> and <b>Lastname</b> fields, otherwise some <b>features</b> of LePress are <b>disabled</b>', lepress_textdomain).'. <a href="'.admin_url().'profile.php'.'">'.__('Edit profile', lepress_textdomain).'</a>');
		} else {
			if(!$access_granted) {
				//Not blog owner, maybe needed in future
			}
		}
	}
	
	function echoNoticeDiv($str, $error = false) {
		$class = 'updated';
		if($error) {
			$class = 'error';
		}
		echo '<div id="message" class="'.$class.'"><p>'.$str.'</p></div>';
	}
	
	function safeStringInput($str, $strip_tags = false) {
		if($strip_tags) {
			return trim(strip_tags($str));
		} else {
			return htmlspecialchars($str, ENT_COMPAT, $this->getBlogCharset(), false);
		}
	}
	
	/* Fail-safe blog charset retriever */
	
	function getBlogCharset() {
		return get_option('blog_charset', 'UTF-8');
	}
    function isStudentFeatures() {
    	$user_roles = $this->get_option('user-role');
    	return isSet($user_roles['student']) ? $user_roles['student'] : false;
    }

    function isTeacherFeatures() {
    	$user_roles = $this->get_option('user-role');
    	return isSet($user_roles['teacher']) ? $user_roles['teacher'] : false;
    }
    
    function getDisplayNames() {
    	$set = $this->get_option('display-names');
    	return $set ? $set : 1;
    }

    function user_role_input() {
    	echo '<input type="checkbox" name="lepress-settings[user-role][student]" '.($this->isStudentFeatures() == 2 && $this->isStudentFeatures() != NULL ? 'checked=checked':'').' id="lepress-settings[user-role][student]" value="2"/> <label for="lepress-settings[user-role][student]">'.__('Student', lepress_textdomain).'</label><br />';
    	echo '<input type="checkbox" name="lepress-settings[user-role][teacher]" '.($this->isTeacherFeatures() == 1 && $this->isTeacherFeatures() != NULL ? 'checked=checked':'').' id="lepress-settings[user-role][teacher]" value="1"/> <label for="lepress-settings[user-role][teacher]">'.__('Teacher', lepress_textdomain).'</label>';
    }
    
    function display_names_input() {
		$setting = $this->getDisplayNames();
		$current_user = $this->getBlogOwnerUser();
		echo '<input type="radio" name="lepress-settings[display-names]" '.($setting == 1 ? 'checked="checked"':'').' value="1" id="display-names-1"/> <label for="display-names-1">'.__('Given name Surname', lepress_textdomain).' ('.trim(($current_user->first_name ? $current_user->first_name : $current_user->user_login).' '.$current_user->last_name).')</label><br />';
		echo '<input type="radio" name="lepress-settings[display-names]" '.($setting == 2 ? 'checked="checked"':'').' value="2" id="display-names-2"/> <label for="display-names-2">'.__('Surname Given name', lepress_textdomain).' ('.trim(($current_user->last_name ? $current_user->last_name : $current_user->user_login).' '.$current_user->first_name).')</label>';
    }

    function generalSectionHeader() {
    	echo '<h3>'.__('General settings', lepress_textdomain).'</h3>';
    }

	//TODO translate this, figure out what to display here ?
    function getWelcomeScreen() {
    	echo "This is our welcome screen!<br />";
    	echo 'Please configure your plugin <a href="options-general.php?page=lepress-settings">here</a>';
    }

    /* Add LePress menu */

    function createOptionsMenus() {
    	$current_user = wp_get_current_user();
    	if(current_user_can('manage-lepress') || is_super_admin() || (get_bloginfo('admin_email') === $current_user->user_email)) {
	    	add_menu_page( 'LePress', 'LePress', 3,'lepress');
	    	if(!$this->isTeacherFeatures() && !$this->isStudentFeatures()) {
	    		add_submenu_page('lepress', __("LePress Manual", lepress_textdomain),__("What is LePress?", lepress_textdomain),3, 'lepress', array(&$this, 'getWelcomeScreen' ));
	    	}
		    add_options_page(__("LePress settings", lepress_textdomain), __("LePress", lepress_textdomain), 'manage_options', 'lepress-settings', array(&$this, 'createDefaultOptionsPage'));
    	}
	}

	/* Generates default options page */

	function createDefaultOptionsPage() {
		echo '<div class="wrap">';
		screen_icon();
		echo '<h2>LePress</h2>';
		echo '<form action="options.php" method="post">';
		settings_fields('lepress-settings');
		do_settings_sections('lepress');
		echo '<input name="Submit" type="submit" value="'.__('Save').'"/>';
		echo '</form></div>';
	}

	/* Get option from lepress-settings options array */

	function get_option($option) {
		$options = get_option('lepress-settings');
    	return $options ? $options[$option] : false;
	}

	/* Plugin deactivation hook, deletes all settings data */

	function uninstall() {
		$wipe_out = $this->get_option('wipe_out_flag');
		/* Init student data remove */
		if($this->isStudentFeatures() || $wipe_out) {
			global $LePressStudent;
			//Always ensure we are just in case trying to wipe student tables too
			//maybe student role was active, then was disabled
			if(!$LePressStudent instanceOf LePressStudentRole) {
				require_once('classes/student.php');
				$LePressStudent = new LePressStudentRole();
			}
			$LePressStudent->initUninstall($wipe_out); //Unsubscribes from all the courses, rest of the data is kept intact;
		}
		
		if($this->isTeacherFeatures() || $wipe_out) {
			global $LePressTeacher;
			//Always ensure we are just in case trying to wipe teacher tables too
			//maybe teacher role was active, then was disabled
			if(!$LePressTeacher instanceOf LePressTeacherRole) {
				require_once('classes/teacher.php');
				$LePressTeacher = new LePressTeacherRole();
			}
			$LePressTeacher->initUninstall($wipe_out); //Remove all subscriptions, rest of the data is kept intact
		}
		
		/* Finally remove main settings data */
		unregister_setting('lepress-settings', 'lepress-settings');
		delete_option('lepress-settings');
	}

	/* Date + timezone fix with i18n  */

	function date_i18n_lepress($tstamp, $to_int = false, $format = false, $time = false) {
	    if($to_int) {
	        $tstamp = intval($tstamp);
	    }
	    $tstamp = !is_numeric($tstamp) ? strtotime($tstamp) : $tstamp;
	    $gmt_offset = get_option('gmt_offset') ? get_option('gmt_offset') : 0;
		return date_i18n((!$format ? get_option('date_format') : $format).' '.($time ? get_option('time_format') :''), (int) $tstamp + ($gmt_offset * 3600));
	}
	
	/* Multibyte cut string */

	function mb_cutstr($str, $end = 200, $start = 0) {
	    $str = strip_tags($str);
	    if(mb_strlen($str, $this->getBlogCharset()) <= $end) {
	        return $str;   
	    }
	    $arr = explode(" ", mb_substr($str, $start, $end, $this->getBlogCharset()));
	    //Pop last element, which is probably half cut word
	    array_pop($arr);
	    return (string) implode(" ", $arr)."...";    
	}
	
	function registerAdvancedSection() {
		add_settings_section('lepress-advanced', '', array(&$this, 'advancedSectionHeader'), 'lepress');
		add_settings_field('lepress-wipe-out', __("Wipe data on plugin deactivation", lepress_textdomain), array(&$this, 'permanentDeleteField'), 'lepress', 'lepress-advanced');
	}
	
	function permanentDeleteField() {
		$wipe_out = $this->get_option('wipe_out_flag');
		echo '<input type="checkbox" name="lepress-settings[wipe_out_flag]" '.($wipe_out ? 'checked="checked"' : '').' id="lepress-settings[wipe_out_flag]" value="1"/> <label for="lepress-settings[wipe_out_flag]"><b>'.__('CAUTION! ALL THE DATA WILL BE DELETED!', lepress_textdomain).'</b></label> <span class="description">'.__('If checked, LePress database tables will be deleted. Data saved as Wordpress metadata will not be altered.', lepress_textdomain).'</span>';
	}
	
	function advancedSectionHeader() {
    	echo '<h3>'.__('Advanced', lepress_textdomain).'</h3>';
    }

}

if(class_exists('LePress')) {
    //Define global variable $LePress
    $LePress = new LePress();

    if($LePress->isTeacherFeatures()) {
		require_once('classes/teacher.php');
		$LePressTeacher = new LePressTeacherRole();
    }
    if($LePress->isStudentFeatures()) {
		require_once('classes/student.php');
		$LePressStudent = new LePressStudentRole();
    }
    
    //Add extra advanced section
    add_action('admin_init', array(&$LePress, 'registerAdvancedSection'));
}

?>