<?php

class LePressService {

	function __construct() {
		add_filter('query_vars', array(&$this, 'service_add_trigger'));
		add_action('template_redirect', array(&$this,'service_trigger_check'));
	}
	
	function service_add_trigger($vars) {
		$vars[] = 'lepress-service';
		return $vars;
	}
	
	function service_trigger_check() {
		if(intval(get_query_var('lepress-service')) == 1) {
			$this->handleRequest(); //This function call ends with exit();
		}
	}

	function getRole() {
		return $this->role ? $this->role : false;
	}

	function getResponseRole() {
		return $this->getRole() == 1 ? 2 : 1;
	}

	function getAction() {
		return $this->action ? $this->action : false;
	}

	function handleRequest() {
		/* IF passed above if condition, start the fun */
		if(isSet($_POST)) {
			$this->role = $_POST['lepress-role'] ? strip_tags($_POST['lepress-role']) : false;
			$this->action = $_POST['lepress-action'] ? strip_tags($_POST['lepress-action']) : false;
		}

		
		if(isSet($_POST['simple-subscriber-blog'])) {
			$my_lepress_password = md5(trim($_POST['lepress-password']));
			$request = new LePressRequest(1, 'simple-subscribe');
			$request->addParam('course_url', trim(strip_tags($_POST['course-url'])));
			$request->addParam('my-lepress-password', $my_lepress_password);
			$blog_url = trim(strip_tags($_POST['simple-subscriber-blog']));
			if(filter_var($blog_url, FILTER_VALIDATE_URL)) {
				$request->doPost($blog_url);
				switch($request->getStatusCode()) {
					case 202:
						echo 1;
						break;
					case 401:
						echo 2;
						break;
					case 500:
						echo 0;
						break;
					default:
						echo -1;
						break;
				}
			} else {
				echo 3;
			}
			exit;
		}
		
		//otherwise parse request
		if($this->getRole() == 1) {
			$this->parseTeacherRequest();
		}
		if($this->getRole() == 2) {
			$this->parseStudentRequest();
		}

		if($this->getRole() == 22) { //Student blog to other student blog
			$this->parseMateRequest();
		}
	}

	function parseMateRequest() {
		global $LePressStudent;
		switch($this->getAction()) {
			case 'inviteMeToGroup':
					$LePressStudent->groups->addMeToGroup($_POST['group_name'], $_POST['group_key'], $_POST['course_url']);
				break;
		}
		exit();
	}


	function parseTeacherRequest() {
		global $LePressStudent;
		switch($this->getAction()) {
			case 'subscription-status-changed':
				$LePressStudent->subscriptions->updateStatus($_POST['course-url'], $_POST['course-status'], $_POST['accept-key'], $_POST['lepress-message']);
				break;
			case 'getAnswerByID':
				if($answer = $LePressStudent->subscriptions->getAnswerByPostID($_POST['post_id'])) {
					echo $answer;
				} else {
					header("HTTP/1.1 404 Not found");
				}
				break;
			case 'addFeedback':
				$LePressStudent->subscriptions->addFeedback($_POST['post_id'], $_POST['teacher_id'], $_POST['accept-key'], $_POST['feedback'], $_POST['grade']);
				break;
			case 'simple-subscribe':
				if($LePressStudent->get_option('lepress-simple-password') == $_POST['my-lepress-password']) {
					$result = $LePressStudent->subscriptions->subscribe($_POST['course_url'], '', true);
					if($result) {
						header("HTTP/1.1 202 Accepted");
					} else {
						header('HTTP/1.1 500 Internal Server Error');
					}
				} else {
					header('HTTP/1.1 401 Unauthorized');
				}
				break;
		}
		exit();
	}

	function parseStudentRequest() {
		if(($cat_id = get_query_var('cat')) > 0 && class_exists('CourseMeta')) {
			global $LePressTeacher;
			$course_meta = new CourseMeta($cat_id, $_POST['accept-key']);
			$blog_url_access = $course_meta->verifyAccess($_POST['lepress-blog'], $_POST['email'], $this->getAction());
			if($blog_url_access > -1) {
				$blog_url = $blog_url_access;
				switch($this->getAction()) {
					case 'subscribe':
						$firstname = $_POST['firstname'];
						$lastname = $_POST['lastname'];
						$email = $_POST['email'];
						$invite_key = isSet($_POST['invite-key']) ? $_POST['invite-key'] : false;
						$message = $_POST['lepress-message'];
						$result = $course_meta->addSubscription($firstname, $lastname, $email, $blog_url, $message, $invite_key);
						if(!$result && is_bool($result)) {
							//Invite key check failed
							header('HTTP/1.1 401 Unauthorized');
						} elseif($result <= -2) {
							header('HTTP/1.1 500 Internal Server Error');
						} else {
							header('HTTP/1.1 202 Accepted');
							echo $result; // Returning course meta to student
						}
						break;
					case 'getCourseMeta';
						echo $course_meta->getCourseMeta();
						break;
					case 'profileUpdated';
						$course_meta->updateStudentProfile($_POST['first_name'], $_POST['last_name'], $_POST['new_email'], $_POST['old_email']);
						header('HTTP/1.1 202 Accepted');
						break;
					case 'unsubscribe':
						$result = $course_meta->removeSubscription($blog_url, $_POST['accept-key']);
						if($result) {
							header('HTTP/1.1 202 Accepted');
						} else {
							header('HTTP/1.1 500 Internal Server Error');
						}
						break;
					case 'getAssignments':
		   				echo $course_meta->getAssignments();
						break;
					case 'getAssignmentByID':
		   				echo $course_meta->getAssignmentByID($_POST['post-id']);
						break;
					case 'addAnswer':
		   				$course_meta->addAnswer($_POST['post_id'], $_POST['answer-url'], $_POST['answer-content'], $_POST['answer-post-id'], $blog_url);
						break;
					case 'getClassmates':
			   			echo $course_meta->getClassmates($blog_url);
						break;
					/* This is disabled, because development of this feature was halted
					 * probably will be completed in near future
					case 'addMyGroup':
						$result = $course_meta->addStudentGroup($_POST['group_name'], $_POST['group_key'], $_POST['email']);
						if($result) {
							header('HTTP/1.1 201 Created');
						} else {
							header('HTTP/1.1 500 Internal Server Error');
						}
						break;
					case 'addMeToGroup':
						$course_meta->addMeToGroup($_POST['group_key'], $blog_url);
						break;
					case 'removeMyGroup':
						$result = $course_meta->removeStudentGroup($_POST['group_key'], $_POST['email']);
						if(!$result) {
							header('HTTP/1.1 500 Internal Server Error');
						}
						break;
						*/
					}
			} elseif($blog_url_access == -1)  {
				header("HTTP/1.1 404 Not found");
			} elseif($blog_url_access == -2) {
				header("HTTP/1.1 409 Conflict");
			} elseif($blog_url_access == -3) {
				header("HTTP/1.1 400 Bad Request");
			} elseif(!$blog_url_access && is_bool($blog_url_access)) {
				header('HTTP/1.1 412 Precondition Failed');
			}
		} else {
			header("HTTP/1.1 404 Not found");
		}
		exit();
	}
}
?>